//EEXCSICR JOB <JOB PARAMETERS>                                                 
//*********************************************************************         
//* Licensed Material - Property of IBM                               *         
//* 5655-P28,5655-S33 (C) Copyright IBM Corp. 2006,2009               *         
//*                                                                   *         
//* All Rights Reserved.                                              *         
//*                                                                   *         
//* U.S. Government users - RESTRICTED RIGHTS - Use, Duplication, or  *         
//* Disclosure restricted by GSA-ADP schedule contract with IBM Corp. *         
//*                                                                   *         
//* Status = HEEX611                                                  *         
//* _________________________________________________________________ *         
//*                                                                   *         
//*   This job creates a new SMP/E consolidated software inventory    *         
//*   (CSI) global zone for WebSphere Extended Deployment for z/OS    *         
//*                                                                   *         
//*  CAUTION: This is neither a JCL procedure nor a complete job.     *         
//*  Before using this job step, you will have to make the following  *         
//*  modifications:                                                   *         
//*                                                                   *         
//*  1) Add the job parameters to meet your system requirements       *         
//*  2) Change #globalcsi to the dataset name of your global CSI      *         
//*     data set                                                      *         
//*  3) Change #hhh to the appropriate high-level qualifier           *         
//*  4) Change #volser to the volser you will be using                *         
//*                                                                   *         
//*  Notes:                                                           *         
//*                                                                   *         
//*  1. This job should complete with a return code 0.                *         
//*                                                                   *         
//*  2. Please review your z/OS data set allocations for the          *         
//*     following datasets                                            *         
//*                                                                   *         
//*       MACLIB                                                      *         
//*                                                                   *         
//*     If the z/OS data set high-level qualifiers are different      *         
//*     from the defaults please change accordingly.                  *         
//*                                                                   *         
//*********************************************************************         
//* This job creates a new global SMPE environment for WebSphere      *         
//* Extended Deployment for z/OS                                      *         
//*********************************************************************         
//* This step allocates the new SMP/E CSI                             *         
//*********************************************************************         
//BUILDCSI EXEC PGM=IDCAMS,REGION=4M                                            
//SYSPRINT DD   SYSOUT=*                                                        
//SMPE     DD   VOL=SER=#volser,UNIT=3390,DISP=SHR                              
//SYSIN    DD   *                                                               
  DEFINE   CLUSTER(                             -                               
                NAME(#globalcsi)                -                               
                FREESPACE(10,5)                 -                               
                KEYS(24 0)                      -                               
                RECORDSIZE(24 143)              -                               
                SHAREOPTIONS(2,3)               -                               
                UNIQUE                          -                               
                VOLUME(#volser) )               -                               
           DATA(                                -                               
                NAME(#globalcsi.DATA)           -                               
                CONTROLINTERVALSIZE(4096)       -                               
                CYLINDERS(25 5))                -                               
           INDEX(                               -                               
                NAME(#globalcsi.INDEX)          -                               
                IMBED                           -                               
                CYLINDERS(2 1))                                                 
/*                                                                              
//*********************************************************************         
//* The RACF group and profile for the new HLQ for the SMPE datasets  *         
//* should already be defined.                                        *         
//*********************************************************************         
//* This step initializes the new CSI                                 *         
//*********************************************************************         
//INITCSI EXEC PGM=IDCAMS,REGION=2M,COND=(0,LT)                                 
//GBLCSI   DD  DSN=#globalcsi,DISP=OLD                                          
//ZPOOL    DD  DSN=SYS1.MACLIB(GIMZPOOL),DISP=SHR                               
//SYSPRINT DD  SYSOUT=*                                                         
//SYSIN    DD  *                                                                
  REPRO OUTFILE(GBLCSI) INFILE(ZPOOL)                                           
/*                                                                              
//*********************************************************************         
//* This step allocates the base set of SMPE datasets.                *         
//*********************************************************************         
//SMPEDS  EXEC PGM=IEFBR14                                                      
//SMPLOG   DD  DSN=#hhh.SMPLOG,DISP=(NEW,CATLG,DELETE),                         
//             DCB=(RECFM=VB,LRECL=510,BLKSIZE=3200),                           
//             UNIT=3390,VOL=SER=#volser,                                       
//             SPACE=(CYL,(5,2))                                                
//SMPLOGA  DD  DSN=#hhh.SMPLOGA,DISP=(NEW,CATLG,DELETE),                        
//             DCB=(RECFM=VB,LRECL=510,BLKSIZE=3200),                           
//             UNIT=3390,VOL=SER=#volser,                                       
//             SPACE=(CYL,(5,2))                                                
//SMPPTS   DD  DSN=#hhh.SMPPTS,DISP=(NEW,CATLG,DELETE),                         
//             DCB=(RECFM=FB,LRECL=80,BLKSIZE=6160),                            
//             UNIT=3390,VOL=SER=#volser,                                       
//             SPACE=(TRK,(8000,800,80))                                        
//*                                                                             
//*********************************************************************         
//* This step creates the options entry                               *         
//*********************************************************************         
//OPTIONS EXEC PGM=GIMSMP,REGION=0M,COND=(0,LT)                                           
//SMPCSI   DD  DSN=#globalcsi,DISP=SHR                                          
//SMPLOG   DD  DSN=#hhh.SMPLOG,DISP=MOD                                         
//SMPLOGA  DD  DSN=#hhh.SMPLOGA,DISP=MOD                                        
//SMPPTS   DD  DSN=#hhh.SMPPTS,DISP=SHR                                         
//SMPOUT   DD  SYSOUT=*                                                         
//SYSPRINT DD  SYSOUT=*                                                         
//SMPCNTL  DD  *                                                                
  SET BDY(GLOBAL) .                                                             
   UCLIN.                                                                       
    ADD OPTIONS(ZOSOPT)                                                         
     RETRYDDN(ALL)                                                              
     DSPREFIX(#hhh)                                                             
     DSSPACE(5000,1000,1300)                                                    
      ASM(ASMA90)                                                               
      AMS(IDCAMS)                                                               
      COPY(IEBCOPY)                                                             
      HFSCOPY(BPXCOPY)                                                          
      COMP(IEBCOPY)                                                             
      RETRY(IEBCOPY)                                                            
      LKED(HEWLH096)                                                            
      ZAP(IMASPZAP)                                                             
      UPDATE(IEBUPDTE).                                                         
   ENDUCL.                                                                      
/*                                                                              
//*********************************************************************         
//* This step creates the global zone zoneindex entry                 *         
//*********************************************************************         
//GLOBAL  EXEC PGM=GIMSMP,REGION=0M                                                     
//SMPCSI   DD  DSN=#globalcsi,DISP=SHR                                          
//SMPLOG   DD  DSN=#hhh.SMPLOG,DISP=MOD                                         
//SMPLOGA  DD  DSN=#hhh.SMPLOGA,DISP=MOD                                        
//SMPPTS   DD  DSN=#hhh.SMPPTS,DISP=SHR                                         
//SMPOUT   DD  SYSOUT=*                                                         
//SYSPRINT DD  SYSOUT=*                                                         
//SMPCNTL  DD  *                                                                
  SET      BDY(GLOBAL) .                                                        
   UCLIN.                                                                       
    ADD GZONE                                                                   
     SREL(Z038)                                                                 
     OPTIONS(ZOSOPT)                                                            
    ZDESC(GLOBAL Zone for EEX 6.1.1).                                           
   ENDUCL.                                                                      
/*                                                                              
//*********************************************************************         
//* This step creates the utility entries                             *         
//*********************************************************************         
//UTILS   EXEC PGM=GIMSMP,REGION=0M,COND=(4,LT)                                           
//SMPCSI   DD  DSN=#globalcsi,DISP=SHR                                          
//SMPLOG   DD  DSN=#hhh.SMPLOG,DISP=MOD                                         
//SMPLOGA  DD  DSN=#hhh.SMPLOGA,DISP=MOD                                        
//SMPPTS   DD  DSN=#hhh.SMPPTS,DISP=SHR                                         
//SMPOUT   DD  SYSOUT=*                                                         
//SYSPRINT DD  SYSOUT=*                                                         
//SMPCNTL  DD  *                                                                
  SET BDY(GLOBAL).                                                              
   UCLIN.                                                                       
    ADD UTILITY(ASMA90  ) NAME(ASMA90  ) RC(0) PRINT(ASMOUT)                    
         PARM(XREF(SHORT),NOLIST,DECK,NOOBJECT).                                
    ADD UTILITY(HEWLH096) NAME(HEWLH096) RC(4) PRINT(LKEDOUT)                   
         PARM(SIZE=(1526K,100K),NCAL,LIST,LET,XREF).                            
    ADD UTILITY(IDCAMS  ) NAME(IDCAMS  ) RC(0) PRINT(VSAMOUT).                  
    ADD UTILITY(IEBCOPY ) NAME(IEBCOPY ) RC(0) PRINT(COPYOUT).                  
    ADD UTILITY(BPXCOPY ) NAME(BPXCOPY ) RC(0) PRINT(SYSPRINT).                 
    ADD UTILITY(IEBUPDTE) NAME(IEBUPDTE) RC(0) PRINT(UPDTOUT).                  
    ADD UTILITY(IEWL    ) NAME(IEWL    ) RC(4) PRINT(LKEDOUT)                   
         PARM(SIZE=(1526K,100K),NCAL,LIST,LET,XREF).                            
    ADD UTILITY(IMASPZAP) NAME(IMASPZAP) RC(0) PRINT(ZAPOUT).                   
   ENDUCL.                                                                      
/*                                                                              
//*********************************************************************         
//* This step defines all the necessary base DDDEF entires            *         
//*********************************************************************         
//DDDEFS  EXEC PGM=GIMSMP,REGION=0M,COND=(4,LT)                                           
//SMPCSI   DD  DSN=#globalcsi,DISP=SHR                                          
//SMPLOG   DD  DSN=#hhh.SMPLOG,DISP=MOD                                         
//SMPLOGA  DD  DSN=#hhh.SMPLOGA,DISP=MOD                                        
//SMPPTS   DD  DSN=#hhh.SMPPTS,DISP=SHR                                         
//SMPOUT   DD  SYSOUT=*                                                         
//SYSPRINT DD  SYSOUT=*                                                         
//SMPCNTL  DD  *                                                                
  SET BDY(GLOBAL).                                                              
   UCLIN.                                                                       
    ADD DDDEF(SMPTLIB)  VOLUME(#volser) UNIT(3390).                             
    ADD DDDEF(SMPPTS )  DA(#hhh.SMPPTS) SHR                                     
        UNIT(3390) VOLUME(#volser).                                             
    ADD DDDEF(SMPDEBUG) DA(#hhh.SMPDEBUG) SHR                                   
        UNIT(3390) VOLUME(#volser).                                             
    ADD DDDEF(SMPLTS )  DA(#hhh.SMPLTS) SHR                                     
        UNIT(3390) VOLUME(#volser).                                             
    ADD DDDEF(SMPLOG )  DA(#hhh.SMPLOG) MOD                                     
        UNIT(3390) VOLUME(#volser).                                             
    ADD DDDEF(SMPLOGA)  DA(#hhh.SMPLOGA) MOD                                    
        UNIT(3390) VOLUME(#volser).                                             
    ADD DDDEF(SMPWRK1 ) SPACE(120,120)  DIR(10)  TRACKS.                        
    ADD DDDEF(SMPWRK2 ) SPACE(30,30)    DIR(10)  TRACKS.                        
    ADD DDDEF(SMPWRK3 ) SPACE(9500,950) DIR(150) TRACKS.                        
    ADD DDDEF(SMPWRK4 ) SPACE(80,40)    DIR(10)  TRACKS.                        
    ADD DDDEF(SMPWRK6 ) SPACE(4100,410) DIR(100) TRACKS.                        
    ADD DDDEF(SYSUT1  ) SPACE(7500,750)          TRACKS.                        
    ADD DDDEF(SYSUT2  ) SPACE(30,30)             TRACKS.                        
    ADD DDDEF(SYSUT3  ) SPACE(30,30)             TRACKS.                        
    ADD DDDEF(SYSUT4  ) SPACE(30,30)             TRACKS.                        
    ADD DDDEF(COPYOUT)  SYSOUT(*).                                              
   ENDUCL.                                                                      
/*                                                                              
//                                                                              
