import sys
lineSeparator = java.lang.System.getProperty("line.separator")
fileSeparator = java.lang.System.getProperty("file.separator")


#----------------------------
# print help 
#----------------------------
def printHelp():
	print """
	Show help:
		>> wsadmin -lang jython -f configureCGSharedLib.py -help
	
	Configure shared library to all target:
		>> wsadmin -lang jython -f configCGSharedLib.py -sharedLibraryName libName -sharedLibraryPath sharedLibpath

		Example:
		wsadmin -lang jython -f configCGSharedLib.py -sharedLibrayName spiSharedLib -sharedLibraryPath c:/sharedLib/spi.jar

	You may have to modify wsadmin to wsadmin.sh or wsadmin.bat, depending upon your operating environment
	"""

def getArgs():

	#print "in getArgs"
	i = 0
	sharedLibName = ""
	sharedLibPath = ""

	while i < len(sys.argv):
		if sys.argv[i] == "-sharedLibraryName":
			i += 1
			sharedLibName = sys.argv[i]
			#print "sharedLibName = " + sharedLibName
			
		elif sys.argv[i] == "-sharedLibraryPath":
			i += 1
			sharedLibPath = sys.argv[i]
			#print "sharedLibPath = " + sharedLibPath			
		else:
			print "unrecognized argument: " + sys.argv[i]

		i += 1

	return (sharedLibName, 	sharedLibPath)	
	
def getTargets():
	#print "Getting GEE targets..."
	targetList = []
	try:
		list = AdminConfig.list('Deployment')
        	results = list.split(lineSeparator)
		for result in results: 
            		appls = result.split("(")        
			name = appls[0].split("_")
			if name[0] == "GEE":
	        		output = AdminApp.view(appls[0], ["-MapModulesToServers"])
            			lines = output.split(lineSeparator)
		            	for line in lines:
					server = line.split(":  ")
					if server[0] == "Server":
						targets = server[1].split("+")
						for target in targets:
							targetList.append(target[10:len(target)])
		                   		break
	except:
		pass

	#print "Getting LongRunningScheduler targets..."

	try:
		output = AdminApp.view("LongRunningScheduler", ["-MapModulesToServers"])
		lines = output.split(lineSeparator)	
		for line in lines:
			server = line.split(":  ")	    
			if server[0] == "Server":
				targets = server[1].split("+")	 		
				for target in targets:
					targetList.append(target[10:len(target)])
				break
	except:
		pass

	#print targetList        
	
	return targetList


#---------------------------
# get cluster
#---------------------------
def getClusterList():
	print "Getting cluster list "
	
	targets = getTargets()
	
	clusterList = []
	for target in targets:
		servers = target.split(",")
		for server in servers:
			type = server.split("=")
			if type[0] == "cluster":
				clusterList.append(type[1])

	return clusterList

def getServerList():
	#print "Getting cluster list "
	serverList = []
	
	targets = getTargets()
	for target in targets:
		servers = target.split(",")
		sName = ""
		nName = ""
		for server in servers:
			type = server.split("=")
			if type[0] == "node":
				nName = type[1]
			elif type[0] == "server":
				sName = type[1]
		if sName != "":
			serverList.append((nName, sName))

	return serverList


def getDynamicClusterMembers(dynamicClusterName):
	endpointList = []
	 
	clusterId = AdminConfig.getid("/ServerCluster:"+dynamicClusterName+"/")
	clusterMembers = AdminConfig. list("ClusterMember", clusterId).split(lineSeparator)
	for clusterMember in clusterMembers:      
		nodeName = AdminConfig.showAttribute(clusterMember, "nodeName")
		serverName = AdminConfig.showAttribute(clusterMember, "memberName")
		print "serverName =" + serverName
		endpointList.append((nodeName,serverName))


	return endpointList


def configSharedLib(sharedLibPath, sharedLibName, cellName, nodeName, serverName):
	

	#1.Identify the server and assign it to the server variable
	tmp = "/Cell:" + cellName + "/Node:" + nodeName + "/Server:" + serverName + "/"
	
	print "configSharedLib for " + tmp

	serv = AdminConfig.getid(tmp)
	#print serv

	#2.Create the shared library in the server
	library = AdminConfig.create('Library', serv, [['name', sharedLibName], ['classPath', sharedLibPath]])
	#print library
	
	#3.Identify the application server from the server and assign it to the appServer variable 
	appServer = AdminConfig.list('ApplicationServer', serv)
	#print appServer
	
	#4. Identify the class loader in the application server and assign it to the classLoader variable
	classLoad = AdminConfig.showAttribute(appServer, 'classloaders')
	cleanClassLoaders = classLoad[1:len(classLoad)-1]
	if cleanClassLoaders == "":
		#	
		#4.1 case no existing class loader. create new one
		#
		print "No existing class loader for this server. Create one..."
		classLoader1 = AdminConfig.create('Classloader', appServer, [['mode',  'PARENT_FIRST']])
		#
		#5. Associate the shared library that you created with the application server through the class loader.
		#
		AdminConfig.create('LibraryRef', classLoader1, [['libraryName', sharedLibName], ['sharedClassloader', 'true']])
	else:
		#
		#4.2 case using existing class loader. 	
      	#    
		print "There's an existing class loader for this server..."		
		classLoader1 = cleanClassLoaders.split(' ')[0]
		#
		#5.Associate the shared library that you created with the application server through the class loader.
		#
		AdminConfig.create('LibraryRef', classLoader1, [['libraryName', sharedLibName], ['sharedClassloader', 'true']])
	#print "done configSharedLib"

#----------------------------
# Config for cluster servers 
# and standalone server
#----------------------------
def configureSharedLibForAllCGServers(sharedLibPath, sharedLibName):
	#assume there's only 1 cell
	cellObj = AdminConfig.list("Cell")
	cName = AdminConfig.showAttribute(cellObj, "name")
	
	#server clusters
	clusters = getClusterList()

	for cluster in clusters:
		#print "cluster name = " + cluster
	
		endpointList = getDynamicClusterMembers(cluster)
		for nName, sName in endpointList:
			#print "nName = " + nName + " sName = " + sName
			configSharedLib(sharedLibPath, sharedLibName, cName, nName, sName)

	#standalone server
	servers = getServerList()
	for nName, sName in servers:
		#print "nName = " + nName + " sName = " + sName
		configSharedLib(sharedLibPath, sharedLibName, cName, nName, sName)

#----------------------------
# Main entry
#----------------------------
if len(sys.argv) > 0:
	option = sys.argv[0]
	if option == "--help":
		printHelp()
	else:
		args = getArgs()	
	
		sharedLibName = args[0]
		sharedLibPath = args[1]
	
		if sharedLibPath == ""  or sharedLibName == "":
			print "Exit"
		else:
			print "Configuring shared library for all CG JobScheduler servers and Endpoint servers..."
			configureSharedLibForAllCGServers(sharedLibPath, sharedLibName)
			print "saving workspace..."
			AdminConfig.save()
   