#=======================================================================================================================================================
#(C) Copyright IBM Corp. 2010 - All Rights Reserved.
#DISCLAIMER:
#The following source code is sample code created by IBM Corporation.
#This sample code is provided to you solely for the purpose of assisting you
#in the  use of  the product. The code is provided 'AS IS', without warranty or
#condition of any kind. IBM shall not be liable for any damages arising out of your
#use of the sample code, even if IBM has been advised of the possibility of
#such damages.
#=======================================================================================================================================================

#=======================================================================================================================================================
# This file is used to migrate Modern Batch FeaturePack 1.0 to WebSphere ComputeGrid 6.1.1.3
# Instructions to use:
# 1. Make sure all jobs have ended and no new jobs are being submitted into the system
# 2. Run: wsadmin.bat -lang jython -f migrateConfig.py --backupConfig [-configFile /tmp/backup.txt]
# 3. Unaugment all the profiles 
# For DMGR : ./manageProfiles.sh -unaugment -profileName Dmgr01 -templatePath = /opt/was/profileTemplates/BATCH/cell/dmgr
# For others: ./manageProfiles.sh -unaugment -profileName AppSrv01 -templatePath = /opt/was/profileTemplates/BATCH/cell/default
# 3.2 and uninstall the FeaturePack
# 4. Drop the LRSCHED tables and delete the datasource
# 5. If you are using a non-Derby DB Create the LRSCHED and LREE tables and their corresponding datasources jdbc/lrsched and jdbc/lree
# 6. Install WCG and Augment all the nodes in the cell and start DMGR
# 7. On the DMGR run: wsadmin.bat -lang jython -f migrateConfig.py --restore [-configFile /tmp/backup.txt] [-lreeJNDIName jdbc/lreeDB2] [-backendId DERBY_V10]
# Note: the last 2 options are required only if you are using a non-Derby database
# Change History
# --------------
# 10-10-10 initial version created
#=======================================================================================================================================================

import sys
import javax.management as mgmt

lineSeparator  = java.lang.System.getProperty("line.separator")
fileSeparator  = java.lang.System.getProperty("file.separator")
wasInstallRoot = java.lang.System.getProperty("was.install.root")
userInstallRoot = java.lang.System.getProperty("user.install.root")
osname         = java.lang.System.getProperty("os.name");
wombatConfigFile   = wasInstallRoot + fileSeparator + "wombatConfigBackup.txt"
pgcNameToTargetMap = {}
geeAppNames = []
serverType = ""
LRSAppName = "LongRunningScheduler"
PGCProxyAppName = "PGCProxyController"
GEEAppName = "GEE"
backendId = "DERBY_V100_2"
GEEDataSourceJNDIName = "jdbc/lree"
GEEDBSchema = "LREESCHEMA"


def getServerType():
    servers = AdminConfig.list("Server").splitlines()
    for serverId in servers:
        serverName = serverId.split("(")[0]
        server = serverId.split("(")[1]  #remove name( from id
        server = server.split("/")
        cell = server[1]
        node = server[3]
        cellId = AdminConfig.getid("/Cell:" + cell + "/")
        cellType = AdminConfig.showAttribute(cellId, "cellType")
        
        #print "cellId: " + cellId
        #print "cellType: " + cellType
        if cellType == "DISTRIBUTED":
            if AdminConfig.showAttribute(serverId, "serverType") == "DEPLOYMENT_MANAGER":
                serverType = "DEPLOYMENT_MANAGER" 
        elif cellType == "STANDALONE":
            if AdminConfig.showAttribute(serverId, "serverType") == "APPLICATION_SERVER":
                serverType = "APPLICATION_SERVER" 
    
        #wasInstallRoot = getWASHome() 
        wombatConfigFile = wasInstallRoot + fileSeparator + "wombatConfigBackup.txt"
        print "serverType: " + serverType
        return serverType
def printHelp():
    print """

    Show help:
    >> wsadmin -lang jython -f migrateConfig.py --help

    Backs up the GEE and LRSCHED configs:
    >> wsadmin -lang jython -f migrateConfig.py --backup [-configFile <configFile>]
      
    Restores the GEE and LRSCHED configs:
    >> wsadmin -lang jython -f migrateConfig.py --restore [-configFile <configFile>] [-backendId <backendId for LREE database]
  
    Valid backendIds include: 
    DB2UDBNT_V82_1, DB2UDBNT_V91_1, DB2UDBOS390_V7_1, DB2UDBOS390_V8_1
    DB2UDBOS390_V9_1, DERBY_V100_2, ORACLE_V9_1, ORACLE_V10_1, ORACLE_V11_1 
    You may have to modify wsadmin to wsadmin.sh or wsadmin.bat, depending upon your operating environment.
  
    """
    


def getPGCTargets():

    targetList = []
 
    try:
        list = AdminConfig.list('Deployment')
        results = list.split(lineSeparator)
        for result in results: 
            appls = result.split("(")        
            name = appls[0].split("_")
            if name[0] == "PGCController":
                output = AdminApp.view(appls[0], ["-MapModulesToServers"])
                lines = output.split(lineSeparator)
                for line in lines:
                    server = line.split(":  ")
                    if server[0] == "Server":
                        targets = server[1].split("+")
                        for target in targets:
                            targetList.append(target[10:len(target)])
                            geeAppNames.append(appls[0])
                        break
    except:
        pass
        
    return targetList


def getPGCAppNames():
    try:
        list = AdminConfig.list('Deployment')
        results = list.split(lineSeparator)
        for result in results: 
            appls = result.split("(")        
            name = appls[0].split("_")
            if name[0] == "PGCController":
                geeAppNames.append(appls[0])
              
    except:
        pass
        
    return geeAppNames    

def isTarget(target):
    targets = getTargets(target)
    for t in targets:
        if t == target:
            return "true"
    return "false"


    
def save():
    AdminConfig.save()
    if serverType == "DEPLOYMENT_MANAGER":
        dmgrbean = AdminControl.queryNames("type=DeploymentManager,*")
        AdminControl.invoke(dmgrbean, "syncActiveNodes", "true")
        msg = "Configuration was saved and synchronized to the active nodes"
    else:
        msg = "Configuration was saved"

    print "INFO: Configuration was saved and synchronized to the active nodes"

  

def removePGCApps():

    print ""

    try:
        list = AdminConfig.list('Deployment')
        results = list.split(lineSeparator)
        for result in results: 
            appls = result.split("(")        
            name = appls[0].split("_")
            if name[0] == "PGCController":
                print AdminApp.uninstall(appls[0])
                print "INFO: Removed Grid Execution Environment (" + appls[0] + ")"
    except:
        pass
        
     
    save()

def getLRSAttributes():
    lrsAttrs = ""
    try:
        lrsAttrs = AdminTask.showLongRunningSchedulerAttributes()
        print "INFO: Long Running Scheduler was installed with following attributes :" + lrsAttrs
    except:
        print "INFO: NO LRS configuration detected"
        pass
     
    return lrsAttrs  
     
     
def removeLRSApps():

    AdminApp.uninstall(LRSAppName)
    
    AdminApp.uninstall(PGCProxyAppName)
        
    
    save()

def saveConfig():

    print ""
    targets = getPGCTargets()
    
    if len(targets) == 0:
        print "INFO: No Grid Execution Environment was installed"
        f = open(wombatConfigFile, "w")
        lrsAttrs = getLRSAttributes()
        if len(lrsAttrs) > 0:    
            f.write(lrsAttrs)
        f.close()
        return

    if len(targets) >= 1:
        print "INFO: Grid Execution Environment was installed on following target: "
        print "      " + targets[0]
        f = open(wombatConfigFile, "w")
        for name, target in zip(geeAppNames, targets):
            f.write(name + ":" + target + "\n")
        lrsAttrs = getLRSAttributes()
        if len(lrsAttrs) > 0:    
            f.write(lrsAttrs)
        f.close()
        return
       
def readSavedConfig():
    print "Reading saved configuration from " + wombatConfigFile
    f = open(wombatConfigFile, "r")        
    lines = f.readlines()
    for line in lines:
        print "Line: " + line
        name = line.split("_") 
        if name[0] == "PGCController":
            tokens = line.split(":")
            
            pgcNameToTargetMap[tokens[0]] = tokens[1]
            print "PGCTarget:" + tokens[0] + ":" + tokens[1]
        else: 
            tokens = line.split("=")
            if tokens[0] == "datasourceJNDIName":
                LRSDataSourceJNDIName = tokens[1]
                print "datasourceJNDIName:" + LRSDataSourceJNDIName
            elif tokens[0] ==  "deploymentTarget":
                LRSDeploymentTarget = line[len("deploymentTarget")+1:len(line)]  
                print "deploymentTarget:" + LRSDeploymentTarget
            elif tokens[0] ==  "endpointJobLogLocation":
                endpointJobLogLocation = tokens[1]
                print "endpointJobLogLocation:" + endpointJobLogLocation       
            elif tokens[0] == "databaseSchemaName":
                LRSDBSchemaName = tokens[1]
                print "databaseSchemaName:" + LRSDBSchemaName           
        
    f.close()
    print "Finished reading configuration"
    return pgcNameToTargetMap, LRSDataSourceJNDIName, LRSDeploymentTarget, endpointJobLogLocation, LRSDBSchemaName
    

def updateGridEndpointDSVariable(newJNDIName):
    print "Updating GRID_ENDPOINT_DATASOURCE to " + newJNDIName
    cell = AdminControl.getCell()
    AdminTask.setVariable(['-variableName', 'GRID_ENDPOINT_DATASOURCE', '-variableValue', newJNDIName, '-scope', 'Cell=' + cell]) 
    save()
    print "Finished Updating GRID_ENDPOINT_DATASOURCE to " + newJNDIName
    
def updateGridEndpointBackendIdVariable(newBackendId):
    print "Updating GRID_ENDPOINT_BACKENDID to " + newBackendId
    cell = AdminControl.getCell()
    AdminTask.setVariable(['-variableName', 'GRID_ENDPOINT_BACKENDID', '-variableValue', newBackendId, '-scope', 'Cell=' + cell]) 
    save()
    print "Finished Updating GRID_ENDPOINT_BACKENDID to " + newBackendId

def restoreLRSAttributes(datasourceJNDIName, lrsDeploymentTarget, endpointJobLogLocation, lrsDBSchemaName):
    print "Restoring LRS Attributes..."
        
    AdminTask.modifyLongRunningSchedulerAttribute('[-name endpointJobLogLocation -value ' + endpointJobLogLocation + ']')
    print "Endpoint job log location set to: " + endpointJobLogLocation
    
    # skip for STANDALONE server as it will be precofigured
    if serverType == "DEPLOYMENT_MANAGER":
        AdminTask.modifyLongRunningSchedulerAttribute('[-name deploymentTarget -value ' + lrsDeploymentTarget + ']')
        print "LRS target set to: " + lrsDeploymentTarget
    
    AdminTask.modifyLongRunningSchedulerAttribute('[-name databaseSchemaName -value ' + lrsDBSchemaName + ']')
    print "LRS DB Schema set to: " + lrsDBSchemaName
    
    AdminTask.modifyLongRunningSchedulerAttribute('[-name datasourceJNDIName -value ' + datasourceJNDIName + ']')
    print "LRS datasource name set to: " + datasourceJNDIName
    
    save()
    
    print "Restored LRS Attributes"
def getWASHome():
    #varMap = AdminConfig.getid("/Cell:" + cell + "/Node:" + node + "/VariableMap:/")
    #entries = AdminConfig.list("VariableSubstitutionEntry", varMap)
    #eList = entries.split(lineSep)
    #for entry in eList:
    #    name =  AdminConfig.showAttribute(entry, "symbolicName")
    #    if name == "WAS_INSTALL_ROOT":
    #        value = AdminConfig.showAttribute(entry, "value")
    #        return value
    #failover
    #return java.lang.System.getenv('WAS_HOME')
    return wasInstallRoot



def deployEndpoint(deploymentTarget):
    print "Deploying GEE to " + deploymentTarget
    gridConfigurator = AdminControl.queryNames("WebSphere:type=GridConfiguratorMBean,process=dmgr,*")
    params = [deploymentTarget, GEEDataSourceJNDIName, GEEDBSchema, "1"]
    sigs = ['java.lang.String','java.lang.String','java.lang.String','java.lang.String']
    AdminControl.invoke_jmx(mgmt.ObjectName(gridConfigurator),'configureGridEndpoint',params,sigs)
    save()
    print "Finished Deploying GEE"

    
def restoreConfig():
    print "INFO: reading saved configuration"
    pgcNameToTargetMap, datasourceJNDIName, lrsDeploymentTarget, endpointJobLogLocation, lrsDBSchemaName = readSavedConfig()    
   
    print "INFO: restoring LongRunning Scheduler"  
    restoreLRSAttributes(datasourceJNDIName.rstrip(), lrsDeploymentTarget.rstrip(), endpointJobLogLocation.rstrip(), lrsDBSchemaName.rstrip())
    serverType = getServerType()
    if serverType == "DEPLOYMENT_MANAGER":
        if GEEDataSourceJNDIName != 'jdbc/lree':
            updateGridEndpointDSVariable(GEEDataSourceJNDIName)
    
        if backendId != 'DERBY_V100_2':
            updateGridEndpointBackendIdVariable(backendId)
    
        for(pgcName, pgcTarget) in pgcNameToTargetMap.items():
            print "INFO: Deploying GEE"
            deployEndpoint("WebSphere:" + pgcTarget.rstrip())
    
    
    print "INFO: Migration Complete!"



def backupConfig():
    
    print "INFO: backing up configuration"
    # shutil.copy2(wasInstallRoot + fileSeparator + "profiles" + fileSeparator + "AppSrv01\config\cells" + fileSeparator + AdminControl.getCell() + "\gridscheduler.xml",wasInstallRoot + fileSeparator + "gridschedulerbackup.xml")
    saveConfig()
    print "INFO: backed up configuration"
    
    removeLRSApps()
        
    print "INFO: Uninstalled LRS"   
    
    removePGCApps()
    print "INFO: Uninstalled PGC"    
  




#=======================================================================================================================================================
#
# Main execution logic:
#
#=======================================================================================================================================================
serverType = getServerType()
if len(sys.argv) > 0:
    option = sys.argv[0]
    if option == "--help":
        printHelp()
    elif option == "--backup":
        if len(sys.argv) == 3:
            if sys.argv[1] == '-configFile':
                wombatConfigFile = sys.argv[2]
        sys.exit(backupConfig())
    elif option == "--restore":
        i = 0
        for arg in sys.argv:
            
            if arg == '-configFile':
                wombatConfigFile = sys.argv[i+1]
            if arg == '-lreeJNDIName':
                GEEDataSourceJNDIName = sys.argv[i+1]
            if arg == '-backendId':
                backendId = sys.argv[i+1]
                
            i = i + 1    
                       
        sys.exit(restoreConfig())
             
else:
    printHelp()
