#=======================================================================================================================================================
#(C) Copyright IBM Corp. 2010 - All Rights Reserved.
#DISCLAIMER:
#The following source code is sample code created by IBM Corporation.
#This sample code is provided to you solely for the purpose of assisting you
#in the  use of  the product. The code is provided 'AS IS', without warranty or
#condition of any kind. IBM shall not be liable for any damages arising out of your
#use of the sample code, even if IBM has been advised of the possibility of
#such damages.
#=======================================================================================================================================================

#=======================================================================================================================================================
# This file is used to migrate CG 6.1.1.3 with ifix PM35389 to WebSphere ComputeGrid 8.0
# Instructions to use:
# Migration is performed in the following order
# 1. Dmgr 2. Schedulers 3. Endpoints
#
# ON DMGR 
# 1. Run: wsadmin.bat -lang jython -f migrateConfig61xTo8.py --backupConfig [-configBackupDir /tmp] [-nameOfProfile dmgrprofilename]
# 2. Stop dmgr and Unaugment dmgr profile 
# For DMGR : ./manageProfiles.sh -unaugment -profileName Dmgr01 -templatePath = /opt/was/profileTemplates/wxdcg_augment/cell/dmgr
# 
# 3. and uninstall the WCG 6113
# 4. Install WCG 8
# 5. Augment dmgr profile
# For DMGR: ./manageProfiles.sh -augment -profileName Dmgr01 -templatePath = /opt/was/profileTemplates/wxdcg_augment/cell/dmgr
# 6. Start DMGR and on the DMGR run: wsadmin.bat -lang jython -f migrateConfig.py --restore [-configBackupDir /tmp] [-lreeJNDIName jdbc/lreeDB2] [-lreeSchema MYSCHEMA] [-nameOfProfile mydmgr]
# 7. Migrate the LRSCHED and LREE databases using the appropriate DDLs
#
# On App Servers
# 1. Stop server and node and Unaugment profile 
# For AppServer : ./manageProfiles.sh -unaugment -profileName AppSrv01 -templatePath = /opt/was/profileTemplates/wxdcg_augment/cell/default
# 
# 3. and uninstall the WCG 6113
# 4. Install WCG 8
# 5. Augment profile
# For AppServer: ./manageProfiles.sh -augment -profileName AppSrv01 -templatePath = /opt/was/profileTemplates/wxdcg_augment/cell/default
# 6. Start Node and on the Node run: wsadmin.bat -lang jython -f migrateConfig.py --restore [-node mynode]
#

# # Change History
# --------------
# 3-10-11 initial version created
#=======================================================================================================================================================

import sys
import os
import javax.management as mgmt
from shutil import copytree
from shutil import copyfile
from shutil import rmtree


lineSeparator  = java.lang.System.getProperty("line.separator")
fileSeparator  = java.lang.System.getProperty("file.separator")
wasInstallRoot = java.lang.System.getProperty("was.install.root")
userInstallRoot = java.lang.System.getProperty("user.install.root")
osname         = java.lang.System.getProperty("os.name");


# ***********************
# backup config
# ***********************


def backupConfig(configBackupDir,profileName):

    print "INFO: Saving configuration"

    configPath = wasInstallRoot + fileSeparator +  "profiles"  + fileSeparator + profileName + fileSeparator + "config" + fileSeparator + "cells" + fileSeparator + cellName
    
    
    # Get list of GEE targets
    geetargets = getGEETargets()
    lrstargets = getLRSTargets()
    # Open backup file
    f = open(configBackupDir + fileSeparator + configFile, "w")
    
    
    if len(geetargets) == 0:
        print "INFO: No Grid Execution Environment was installed"
    else:
        print "INFO: Grid Execution Environment was installed on following target: "
        print "      " + geetargets[0]
        # save the GEE targets
        for name, target in zip(geeAppNames, geetargets):
            f.write(name + ":" + target + "\n")
        
    if len(lrstargets) == 0:
        print "INFO: No Long Running Schedulers installed"
    else:
        print "INFO: Long Running Schedulers were installed on following target: "
        for target in lrstargets:
            print " " + target
            f.write("LRS:" + target + "\n")


    f.close()   
        
      

    backupGridSchedulerConfig(configPath)

    print "INFO: Saved LRS Configuration"  
     
    # backup gridclassification dir 
    backupGridClassificationConfig(configPath)
    
    # backup gridjobclass dir
    backupJobClassConfig(configPath)   

    backupServerIndexes(configPath)

    save()

    
    
    print "INFO: Saved configuration"
   
    return

def backupServerIndexes (configPath):
    print "INFO: Backing up ServerIndexes"
    if os.path.exists(configBackupDir + fileSeparator + "nodes"):
        print "nodes dir exists removing"
        rmtree(configBackupDir + fileSeparator + "nodes")

    os.mkdir(configBackupDir + fileSeparator + "nodes")
    nodeList = AdminConfig.list("Node").split(lineSeparator)
    for nodename in nodeList:
        name = nodename.split("(")[0]
        serverIndexFile = configPath + fileSeparator + "nodes" + fileSeparator + name + fileSeparator + "serverindex.xml"
        os.mkdir(configBackupDir + fileSeparator + "nodes" + fileSeparator + name)
        copyfile(serverIndexFile, configBackupDir + fileSeparator + "nodes" + fileSeparator + name + fileSeparator + "serverindex.xml")
    print "INFO: Backed up Serverindexes"

def backupGridSchedulerConfig(configPath):
    print "INFO: Saving gridscheduler.xml" 
    gridschedulerxmlFile =  configPath + fileSeparator + "gridscheduler.xml";
    copyfile(gridschedulerxmlFile, configBackupDir + fileSeparator + "gridscheduler.xml")
    print "INFO: Saved gridscheduler.xml" 
    return
    
def backupGridClassificationConfig(configPath):
    print "INFO: Saving gridclassification configuration" 
    gridclassificationConfigDirPath =  configPath + fileSeparator + "gridclassification"
    gridclassificationBackupPath = configBackupDir + fileSeparator + "gridclassification"
    if os.path.exists(gridclassificationBackupPath):
        # delete old backup data
        rmtree(gridclassificationBackupPath)

    if os.path.exists(gridclassificationConfigDirPath):
        copytree(gridclassificationConfigDirPath, gridclassificationBackupPath)
        print "INFO: Saved gridclassification configuration" 
    return
    
def backupJobClassConfig(configPath):
    print "INFO: Saving gridjobclass configuration" 
    jobClassConfigDirPath =  configPath + fileSeparator + "gridjobclasses";
    jobClassBackupPath =  configBackupDir + fileSeparator + "gridjobclasses"
    if os.path.exists(jobClassBackupPath):
        # delete old backup data
        rmtree(jobClassBackupPath)

    if os.path.exists(jobClassConfigDirPath):
        copytree(jobClassConfigDirPath, configBackupDir + fileSeparator + "gridjobclasses")
        print "INFO: Saved gridjobclass configuration" 
    return    
        
# get the GEE targets

def getGEETargets():

    targetList = []
    print "INFO: Getting GEE targets"
    try:
        list = AdminConfig.list('Deployment')
  
        results = list.split(lineSeparator)
  
        for result in results: 
  
            appls = result.split("(")  
  
            name = appls[0].split("_")
            if name[0] == "GEE" or name[0] == "PGCController":
                output = AdminApp.view(appls[0], ["-MapModulesToServers"])
                lines = output.split(lineSeparator)
                for line in lines:
                    server = line.split(":  ")
                    if server[0] == "Server":
                        targets = server[1].split("+")
                        for target in targets:
                            targetList.append(target[10:len(target)])
                            geeAppNames.append(appls[0])
                        break
    except:
        print "ERROR: Unexpected error: ", sys.exc_info()[0], sys.exc_info()[1]
        pass
        
    return targetList

def getLRSTargets():

    targetList = []
    print "INFO: Getting LRS targets"
    try:
        list = AdminConfig.list('Deployment')
  
        results = list.split(lineSeparator)
  
        for result in results: 
  
            appls = result.split("(")  
  
            name = appls[0].split("_")
            if name[0] == "LongRunningScheduler":
                output = AdminApp.view(appls[0], ["-MapModulesToServers"])
                lines = output.split(lineSeparator)
                for line in lines:
                    server = line.split(":  ")
                    if server[0] == "Server":
                        targets = server[1].split("+")
                        for target in targets:
                            targetList.append(target[10:len(target)])
                            
                        break
    except:
        print "ERROR: Unexpected error: ", sys.exc_info()[0], sys.exc_info()[1]
        pass
        
    return targetList

def getLRSAttributes():
    lrsAttrs = ""
    try:
        lrsAttrs = AdminTask.showLongRunningSchedulerAttributes()
        print "INFO: Long Running Scheduler was installed with following attributes :" + lrsAttrs
    except:
        print "INFO: NO LRS configuration detected"
        pass
     
    return lrsAttrs  
#-----------------------------------------------------
# Restore Config methods
#-----------------------------------------------------

def restoreConfig(configBackupDir,GEEDataSourceJNDIName, LRSDataSourceJNDIName, GEEDBSchema, profileName, nodeToRestore):

    configPath = wasInstallRoot + fileSeparator +  "profiles"  + fileSeparator + profileName + fileSeparator + "config" + fileSeparator + "cells" + fileSeparator + cellName

    if nodeToRestore != "":
        updateSystemAppWebSphereVariableForNode()
       
    else:    
        print "INFO: reading saved configuration"
        geeNameToTargetMap, lrstargets = readSavedConfig()    
   
        # restore gridscheduler.xml
        restoreGridSchedulerConfig(configPath)
        
        # restore grid classification config
        restoreGridClassificationConfig(configPath)
    
        # restore job class config
        restoreJobClassConfig(configPath)

        # restore server indexes
        restoreServerIndexes(configPath)
    
        # update binaries_url for LRS on dmgr
        updateLRSBinariesURL()

        save()
   
        # install PGCProxyController
        appInstalled = isAppInstalled(PGCProxyAppName, configPath)
        if appInstalled == "false":
            for lrstarget in lrstargets:
                print "INFO: installing PGCProxy on " + lrstarget
                lrscell, lrsnode, lrsserver, lrscluster = parseDeploymentTarget(lrstarget.rstrip())
                install(lrscell, lrscluster, lrsnode, lrsserver, PGCProxyAppName,"PGCProxyControllerEAR", PGCProxyAppName + ".ear")
        else:
            updatePGCProxyBinariesURL()

        
        # need to do an app update to get JMC to work
        redeployLRS()

        print len(geeNameToTargetMap)
        pgcNames = [] 
        for(geeName, geeTarget) in geeNameToTargetMap.items():
            print "INFO: Deploying PGC"
            dcell, dnode, dserver, dcluster = parseDeploymentTarget(geeTarget.rstrip())

            appName = generatePGCAppName(dnode, dserver, dcluster)
            appInstalled = isAppInstalled(appName, configPath)
            pgcNames.append(appName)
            if (appInstalled == "false"):
                
                print "deploying PGC with name " + appName
                install(dcell, dcluster, dnode, dserver, appName, PGCAppName, PGCAppName + ".ear")
                save()
           
                #PGC is already installed just need to update the binaries url

        # WAS incorrectly generates the binaries url to container appName.ear when the ear file name is always PGCController
        # need to go in and manually update the binaries urls

        for pgcName in pgcNames:
            updatePGCBinariesURL(pgcName)
    save()    
    print "INFO: Migration Complete!"


def generatePGCAppName (dnode, dserver, dcluster):
    if dserver != "" and dnode != "":
        appName = "PGCController_" + dnode + "_" + dserver
    else:
        appName = "PGCController_" + dcluster

    return appName

def parseDeploymentTarget(deploymentTarget):
    print "parsing deployment Target"
    dcluster = ""
    dnode = ""
    dserver = ""
    dcell = ""
    tokens = deploymentTarget.split(",")
    for token in tokens:
        tks = token.split("=") 
        for tk in tks:
            if tk == "cluster":
                dcluster = tks[1]
                
            elif tk == "node":
                dnode = tks[1]
            elif tk == "server":
                dserver = tks[1]
            elif tk == "cell" or tk == "WebSphere:cell":
                dcell = tks[1]
    print "parsed deployment target" + dcell + dnode + dserver + dcluster        
    return dcell, dnode, dserver, dcluster  
# while migrating schedulers and endpoints set the CG_SYSTEM_APP_LOCATION to the stack_products location
def updateSystemAppWebSphereVariableForNode():
    print "updating websphere variable CG_SYSTEM_APP_LOCATION on" + nodeToRestore
    scope = 'cell='+cellName+',node='+nodeToRestore
    varName = "CG_SYSTEM_APP_LOCATION"
    varValue = "${WAS_INSTALL_ROOT}/stack_products/WCG/systemApps"
    
    print "JOUT: Set variable " + varName + " with value " + varValue + " on node " + node
    AdminTask.setVariable(['-variableName', varName , '-variableValue', varValue, '-scope', scope])
    print "completed updation"

def updateLRSBinariesURL():
    print "Updating binaries URL"
    deploymentXML = configPath + fileSeparator + "applications" + fileSeparator + "LongRunningScheduler.ear" + fileSeparator + "deployments" + fileSeparator + "LongRunningScheduler" + fileSeparator + "deployment.xml"
    file = open(deploymentXML, "r") #Opens the file in read-mode
    depXMLTxt = file.read() #Reads the file and assigns the value to a variable
    file.close() #Closes the file (read session)
    file = open(deploymentXML, "w") #Opens the file again, this time in write-mode
    if depXMLTxt.find("$(WAS_INSTALL_ROOT)/systemApps") != -1 :
        file.write(depXMLTxt.replace("$(WAS_INSTALL_ROOT)/systemApps", "$(CG_SYSTEM_APP_LOCATION)")) #replaces all instances of our keyword
    else:
        file.write(depXMLTxt.replace("$(WAS_INSTALL_ROOT)/feature_packs/BATCH/systemApps", "$(CG_SYSTEM_APP_LOCATION)")) #replaces all instances of our keyword
    # and writes the whole output when done, wiping over the old contents of the file
    file.close() #Closes the file (write session)

    
    print "Succesfully updated " + deploymentXML

def updatePGCProxyBinariesURL ():
    # update pgc proxy controller

    deploymentXML = configPath + fileSeparator + "applications" + fileSeparator + "PGCProxyController.ear" + fileSeparator + "deployments" + fileSeparator + "PGCProxyController" + fileSeparator + "deployment.xml"
    file = open(deploymentXML, "r") #Opens the file in read-mode
    depXMLTxt = file.read() #Reads the file and assigns the value to a variable
    file.close() #Closes the file (read session)
    file = open(deploymentXML, "w") #Opens the file again, this time in write-mode
    if depXMLTxt.find("$(WAS_INSTALL_ROOT)/systemApps") != -1 :
        file.write(depXMLTxt.replace("$(WAS_INSTALL_ROOT)/systemApps", "$(CG_SYSTEM_APP_LOCATION)")) #replaces all instances of our keyword
    else:
        file.write(depXMLTxt.replace("$(WAS_INSTALL_ROOT)/feature_packs/BATCH/systemApps", "$(CG_SYSTEM_APP_LOCATION)")) #replaces all instances of our keyword
    # and writes the whole output when done, wiping over the old contents of the file
    file.close() #Closes the file (write session)
    print "Succesfully updated " + deploymentXML

   
def updatePGCBinariesURL(appName):
    print "Updating PGC binaries URL"
    deploymentXML = configPath + fileSeparator + "applications" + fileSeparator + appName + ".ear" + fileSeparator + "deployments" + fileSeparator + appName + fileSeparator + "deployment.xml"
    file = open(deploymentXML, "r") #Opens the file in read-mode
    depXMLTxt = file.read() #Reads the file and assigns the value to a variable
    file.close() #Closes the file (read session)
    file = open(deploymentXML, "w") #Opens the file again, this time in write-mode
    newDepXMLTxt = ""
    if depXMLTxt.find("$(WAS_INSTALL_ROOT)/systemApps") != -1 :
        newDepXMLTxt = depXMLTxt.replace("$(WAS_INSTALL_ROOT)/systemApps", "$(CG_SYSTEM_APP_LOCATION)") #replaces all instances of our keyword
    else:
        newDepXMLTxt = depXMLTxt.replace("$(WAS_INSTALL_ROOT)/feature_packs/BATCH/systemApps", "$(CG_SYSTEM_APP_LOCATION)") #replaces all instances of our keyword
    # and writes the whole output when done, wiping over the old contents of the file
    file.write(newDepXMLTxt.replace(appName + ".ear", "PGCController.ear")) #replaces all instances of our keyword
    # and writes the whole output when done, wiping over the old contents of the file
    file.close() #Closes the file (write session)
    print "Succesfully updated " + deploymentXML
def readSavedConfig():
    print "Reading saved configuration from " + configBackupDir + fileSeparator + configFile
    GeeNameToTargetMap = {}
    lrstargets = []
    f = open(configBackupDir + fileSeparator + configFile, "r")        
    lines = f.readlines()
    for line in lines:
        print "Line: " + line
                
        if line.startswith("GEE"):
            tokens = line.split(":")
            
            GeeNameToTargetMap[tokens[0]] = tokens[1]
            
            print "GEE:" + tokens[0] + ":" + tokens[1]
        if line.startswith("LRS"):
            tokens = line.split(":")
            lrstargets.append(tokens[1])
        if line.startswith("PGCController"):
             tokens = line.split(":")
             
             GeeNameToTargetMap[tokens[0]] = tokens[1]
             print "PGCController:" + tokens[0] + ":" + tokens[1]
            
        
    f.close()
    print "Finished reading configuration"
    return GeeNameToTargetMap, lrstargets
    
def restoreGridSchedulerConfig(configPath):
    print "Restoring gridscheduler.xml" 
    gridschedulerxmlFile =  configPath + fileSeparator + "gridscheduler.xml";
    copyfile(configBackupDir + fileSeparator + "gridscheduler.xml", gridschedulerxmlFile)
    print "Restored gridscheduler.xml" 
    return
    
def restoreGridClassificationConfig(configPath):
    print "Restoring gridclassification configuration" 
    backupPath = configBackupDir + fileSeparator + "gridclassification"
    gridclassificationConfigDirPath =  configPath + fileSeparator + "gridclassification";
    if os.path.exists(backupPath):
        rmtree(gridclassificationConfigDirPath)
        copytree(backupPath,gridclassificationConfigDirPath)
    print "Restored gridclassification configuration" 
    return
    
def restoreJobClassConfig(configPath):
    print "Restoring gridjobclass configuration" 
    jobClassConfigDirPath =  configPath + fileSeparator + "gridjobclasses";
    backupPath = configBackupDir + fileSeparator + "gridjobclasses"
    if os.path.exists(backupPath):
        rmtree(jobClassConfigDirPath)
        copytree(backupPath, jobClassConfigDirPath)
    print "Restored gridclassification configuration" 
    return    

def restoreServerIndexes (configPath):
    print "INFO: Restoring up ServerIndexes"
    nodeList = AdminConfig.list("Node").split(lineSeparator)
    for nodename in nodeList:
        name = nodename.split("(")[0]
        serverIndexFile = configPath + fileSeparator + "nodes" + fileSeparator + name + fileSeparator + "serverindex.xml"
        copyfile(configBackupDir + fileSeparator + "nodes" + fileSeparator + name + fileSeparator + "serverindex.xml", serverIndexFile )

    print "INFO: Restored up Serverindexes"
        
#--------------------------------------------------------
# Helper methods
#--------------------------------------------------------

# Get cell name

def getCellName():
	cell = AdminConfig.getid('/Cell:/')
	name = AdminConfig.showAttribute(cell,"name")
	return name
	
	
# get server type	

def getServerInfo():
    serverType = ""
    server = ""
    cell = ""
    node = ""
    servers = AdminConfig.list("Server").splitlines()
    for serverId in servers:
        serverName = serverId.split("(")[0]
        server = serverId.split("(")[1]  #remove name( from id
        server = server.split("/")
        cell = server[1]
        node = server[3]
        cellId = AdminConfig.getid("/Cell:" + cell + "/")
        cellType = AdminConfig.showAttribute(cellId, "cellType")
        
        #print "cellId: " + cellId
        #print "cellType: " + cellType
        if cellType == "DISTRIBUTED":
            if AdminConfig.showAttribute(serverId, "serverType") == "DEPLOYMENT_MANAGER":
                serverType = "DEPLOYMENT_MANAGER" 
        elif cellType == "STANDALONE":
            if AdminConfig.showAttribute(serverId, "serverType") == "APPLICATION_SERVER":
                serverType = "APPLICATION_SERVER" 
    
        
    return serverType, server, cell, node
def printHelp():
    print """

    Show help:
    >> wsadmin -lang jython -f migrateConfigTo8.py --help

    Backs up the GEE and LRSCHED configs:
    >> wsadmin -lang jython -f migrateConfigTo8.py --backup [-configBackupDir <config backup dir> -nameOfProfile <dmgr profilename>]
      
    Restores the GEE and LRSCHED configs:
    >> wsadmin -lang jython -f migrateConfigTo8.py --restore [-configBackupDir <config backup dir> -lreeJNDIName <jndi name for LREE>
        -lreeSchema <GEE schema name> -nameOfProfile <dmgr profile name> -node <node to be migrated (only for non-dmgr nodes)]
     
    You may have to modify wsadmin to wsadmin.sh or wsadmin.bat, depending upon your operating environment.
  
    """
    
#------------------------------------------------------------------------------
# add WebSphere Variable
#------------------------------------------------------------------------------
def createCellVariable(varName, varValue, varDesc):
  # check if something of the same name already exists
  varEntries_str = AdminConfig.showAttribute(getCellVariables(),"entries")
  if (len(varEntries_str) > 2):
       varEntries_str =  varEntries_str[1:len(varEntries_str)-1] 
       varEntries = varEntries_str.split(" ")
       for var in varEntries:
          name = AdminConfig.showAttribute(var,"symbolicName")
          if(name == varName):
              print "Variable "+varName+" already exists, skipping creation."
              return
       AdminConfig.create("VariableSubstitutionEntry",getCellVariables(),[["symbolicName",varName],["value",varValue],["description",varDesc]])
  print "Variable "+varName+" created."
  
  return 

def getCellVariables():
  cellName = getCellName()
  varMap = AdminConfig.getid('/Cell:'+cellName+'/VariableMap:/')
  return varMap


#------------------------------------------------------------------------------
# add WebSphere Variable at Node
#------------------------------------------------------------------------------
def createNodeVariable(varName, varValue, varDesc, nodeName):
  print "createNodeVar" + nodeName
  cellName = getCellName()
  
  varMap = AdminConfig.getid("/Cell:" + cellName + "/Node:" + nodeName + "/VariableMap:/")
  
  entries = AdminConfig.list("VariableSubstitutionEntry", varMap)
  
  eList = entries.splitlines()
  
  for entry in eList:
     name = AdminConfig.showAttribute(entry, "symbolicName")
     
     if name == varName:
        print "Variable "+varName+" already exists, skipping creation."
        return
    
  AdminConfig.create("VariableSubstitutionEntry",varMap,[["symbolicName",varName],["value",varValue],["description",varDesc]])
  print "Variable "+varName+" created."
  
  return        


def isTarget(target):
    targets = getTargets(target)
    for t in targets:
        if t == target:
            return "true"
    return "false"


# save configuration
    
def save():
    AdminConfig.save()
    if serverType == "DEPLOYMENT_MANAGER":
        dmgrbean = AdminControl.queryNames("type=DeploymentManager,*")
        AdminControl.invoke(dmgrbean, "syncActiveNodes", "true")
        msg = "Configuration was saved and synchronized to the active nodes"
    else:
        msg = "Configuration was saved"

    print "INFO: Configuration was saved and synchronized to the active nodes"


def redeployLRS():

    print "Performing App update on LRS..."
    systemApps  = wasInstallRoot + fileSeparator + "stack_products" + fileSeparator + "WCG" + fileSeparator + "systemApps"
    earPath     = systemApps + fileSeparator + LRSAppName + ".ear"

    print "Redeploying " + LRSAppName + "..."
    AdminApp.update(LRSAppName,'app','[-operation update -contents ' + earPath + ']')

    print "LRS update completed"
    
def isAppInstalled (appName, configPath):
    foundApp = "false"
    depXML = configPath + fileSeparator + "applications" + fileSeparator + appName + ".ear" + fileSeparator + "deployments" + fileSeparator + appName + fileSeparator + "deployment.xml"
    if os.path.exists(depXML) :
        foundApp = "true"
    #for app1 in AdminApp.list().split():
    #    print "checking app:" + app1
   	#    if ( app1 == appName ) :
    #        print "Application " + appName + " is already installed"
   	#	    foundApp = "true"

    return foundApp
#------------------------------------------------------------------------------
# Install the application
#------------------------------------------------------------------------------

def install(cell, cluster, node, server, appname, webModuleName, earName):

    print "installing :" + appname + " on cell:" + cell + " cluster: " + cluster + " earName: " + earName + " webmodule" + webModuleName
    warName     = webModuleName + ".war"
    xmlName     = warName + ",WEB-INF/web.xml"
    
    cellName    = "WebSphere:cell=" + cell
    systemApps  = "$(CG_SYSTEM_APP_LOCATION)"
    earPath     = wasInstallRoot + fileSeparator + "stack_products" + fileSeparator + "WCG" + fileSeparator + "systemApps" + fileSeparator + earName
    

    if cluster != "":
        msg = "cluster " + cluster
        installTarget         = [ "-cluster", cluster ]
        MapModulesToServers   = [ [".*", ".*", cellName + ",cluster=" + cluster ] ]

    else:
        msg = "node/server " + node + "/" + server
        installTarget         = [ "-node", node, "-server", server ]
        MapModulesToServers   = [ [".*", ".*", cellName + ",node=" + node + ",server=" + server ] ]


    
    #uninstall(cluster,appname)
    

    options = [ 
                "-nopreCompileJSPs",
                "-distributeApp",
                "-noreloadEnabled",
                "-noprocessEmbeddedConfig",
                "-usedefaultbindings",
                "-zeroEarCopy",

                installTarget,

    
                "-MapModulesToServers",             MapModulesToServers,
    

                "-appname",                         appname, 
                "-installed.ear.destination",       systemApps
              ]
    
    AdminApp.install(earPath, options)
    
    
    
    print "INFO: installed " + appname + " to " + msg 



#------------------------------------------------------------------------------
# Uninstall the PGCProxyController
#------------------------------------------------------------------------------

def uninstall(cluster,appname):
    try: 
        AdminApp.uninstall(appname)
        result = "removed"
    except:
        result = ""

    
#=======================================================================================================================================================
#
# Main execution logic:
#
#=======================================================================================================================================================

configFile   = "cgConfigBackup.txt"
configBackupDir = wasInstallRoot + fileSeparator + "temp"
configPath = ""
cellName = ""
server = ""
node = ""
geeNameToTargetMap = {}
geeAppNames = []
serverType = ""
LRSAppName = "LongRunningScheduler"
PGCAppName = "PGCController"
PGCProxyAppName = "PGCProxyController"
GEEAppName = "GEE"
GEEDataSourceJNDIName = "jdbc/lree"
LRSDataSourceJNDIName = "jdbc/lrsched"
GEEDBSchema = "LREESCHEMA"
profileName = "Dmgr01"
nodeToRestore = ""


serverType, server, cellName, node = getServerInfo()
    
configPath = wasInstallRoot + fileSeparator +  "profiles"  + fileSeparator + profileName + fileSeparator + "config" + fileSeparator + "cells" + fileSeparator + cellName

if len(sys.argv) > 0:
    option = sys.argv[0]
    if option == "--help":
        printHelp()
    elif option == "--backup":
        i = 1
        while i < len(sys.argv):
            
            if sys.argv[i] == '-configBackupDir':
                configBackupDir = sys.argv[i+1]
            if sys.argv[i] == '-nameOfProfile':
                profileName = sys.argv[i+1]     
            i = i + 2    
            
        sys.exit(backupConfig(configBackupDir,profileName))
    elif option == "--restore":
        i = 1
        while i < len(sys.argv):
            
            if sys.argv[i] == '-configBackupDir':
                
                configBackupDir = sys.argv[i+1]
            if sys.argv[i] == '-lreeJNDIName':
                GEEDataSourceJNDIName = sys.argv[i+1]
            if sys.argv[i] == '-lrsJNDIName':
                LRSDataSourceJNDIName = sys.argv[i+1]
            if sys.argv[i] == '-lreeSchema':
                GEEDBSchema = sys.argv[i+1]  
            if sys.argv[i] == '-nameOfProfile':
                profileName = sys.argv[i+1]
            if sys.argv[i] == '-node':
                nodeToRestore = sys.argv[i+1]
            i = i + 2    
                       
        sys.exit(restoreConfig(configBackupDir,GEEDataSourceJNDIName, LRSDataSourceJNDIName, GEEDBSchema, profileName, nodeToRestore))
             
else:
    printHelp()
