import sys
import java.lang.System  as  jsys

lineSeparator  = java.lang.System.getProperty("line.separator")  
fileSeparator  = java.lang.System.getProperty("file.separator")

scriptName         = "redeployGEE.py"

defaultAppName     = "GEE"


def printHelp():
  print """

  Show help:
      >> wsadmin -lang jython -f redeployGEE.py --help

  Redeploy GEE:
      >> wsadmin -lang jython -f redeployGEE.py <GEE backend ID>

      Example:
      >> wsadmin -lang jython -f redeployGEE.py DB2UDBOS390_V9_1
  
  You may have to modify wsadmin to wsadmin.sh or wsadmin.bat, depending upon your operating environment.
  
  """


def getTargets():

    targetList = []
    count = 0
    geeAppName = ""

    try:
        list = AdminConfig.list('Deployment')
        results = list.split(lineSeparator)
        for result in results: 
           appls = result.split("(")        
           name = appls[0].split("_")
           if name[0] == "GEE":
              targetList.append(appls[0])
    except:
        pass
        
    return targetList



def listTargets():

    print ""
    targets = getTargets()
    
    if len(targets) == 0:
        print "INFO: No Grid Execution Environment Application was found"
        return 


    if len(targets) == 1:
       print "INFO: The following Grid Execution Environment Application was found: "
       print "      " + targets[0]
       return
    
    print "INFO: The following Grid Execution Environment Applications were found: "
    for target in targets:
         print "      " + target

#------------------------------------------------------------------------------
# Redeploy the application
#------------------------------------------------------------------------------

def redeploy(targets,backend):

    wasHome     = java.lang.System.getenv('WAS_HOME')
    systemApps  = wasHome + fileSeparator + "systemApps"
    earPath     = systemApps + fileSeparator + defaultAppName + ".ear"

    for target in targets:
        print "Redeploying " + target.rstrip() + "..."
        AdminApp.update(target.rstrip(),'app','[-update.ignore.new -operation update -contents ' + earPath + ' -BackendIdSelection [[.* BatchJobExecutionEnvironmentEJBs.jar,.* ' + backend + ']]]')


    print "Saving config..."
    save()


#------------------------------------------------------------------------------
# Print script usage
#------------------------------------------------------------------------------

def usage(msg):
    print ""
    print scriptName + " : " + msg
    print ""
    print "Usage: wsadmin -lang jython -f " + scriptName + " < backend ID  (e.g. DB2UDBOS390_V9_1) >"
    print ""
    sys.exit(101)

#------------------------------------------------------------------------------
# Save the configuration
#------------------------------------------------------------------------------
def save():
    AdminConfig.save()
    dmgrbean = AdminControl.queryNames("type=DeploymentManager,*")
    if dmgrbean != "":
        AdminControl.invoke(dmgrbean, "syncActiveNodes", "true")
        msg = "Configuration was saved and synchronized to the active nodes"
    else:
        msg = "Configuration was saved"

    print scriptName + " INFO: " + msg  

#------------------------------------------------------------------------------
# Main entry point
#------------------------------------------------------------------------------

failOnError = "false" 


if len(sys.argv) > 0:
    option = sys.argv[0]
    if option == "--help":
        printHelp()
    elif option == "--list":
        sys.exit(listTargets())   
    else:
        backend = option   
        targets = getTargets()
        redeploy(targets,backend)
else:

    usage("Backend ID is required")

